@@ -23,7 +23,6 @@
 
 #include "gdkglcontext.h"
 #include "gdkdrawcontextprivate.h"
-#include "gdkmemorytexture.h"
 
 G_BEGIN_DECLS
 
@@ -62,7 +61,7 @@ struct _GdkGLContextClass
 
   GdkGLBackend        backend_type;
 
-  gboolean              (* realize)                             (GdkGLContext          *context,
+  GdkGLAPI              (* realize)                             (GdkGLContext          *context,
                                                                  GError               **error);
 
   gboolean              (* make_current)                        (GdkGLContext          *context,
@@ -100,20 +99,19 @@ gboolean                gdk_gl_backend_can_be_used              (GdkGLBackend
                                                                  GError         **error);
 void                    gdk_gl_backend_use                      (GdkGLBackend     backend_type);
 
-GdkGLContext *          gdk_gl_context_new_for_surface          (GdkSurface      *surface);
+GdkGLContext *          gdk_gl_context_new                      (GdkDisplay      *display,
+                                                                 GdkSurface      *surface);
 
+gboolean                gdk_gl_context_is_api_allowed           (GdkGLContext    *self,
+                                                                 GdkGLAPI         api,
+                                                                 GError         **error);
 void                    gdk_gl_context_set_is_legacy            (GdkGLContext    *context,
                                                                  gboolean         is_legacy);
 
-void                    gdk_gl_context_upload_texture           (GdkGLContext    *context,
-                                                                 const guchar    *data,
-                                                                 int              width,
-                                                                 int              height,
-                                                                 int              stride,
-                                                                 GdkMemoryFormat  data_format,
-                                                                 guint            texture_target);
-GdkGLContextPaintData * gdk_gl_context_get_paint_data           (GdkGLContext    *context);
-gboolean                gdk_gl_context_use_texture_rectangle    (GdkGLContext    *context);
+gboolean                gdk_gl_context_check_version            (GdkGLContext    *context,
+                                                                 int              required_major,
+                                                                 int              required_minor);
+
 gboolean                gdk_gl_context_has_unpack_subimage      (GdkGLContext    *context);
 void                    gdk_gl_context_push_debug_group         (GdkGLContext    *context,
                                                                  const char      *message);
@@ -135,18 +133,6 @@ gboolean                gdk_gl_context_has_debug                (GdkGLContext
 
 gboolean                gdk_gl_context_use_es_bgra              (GdkGLContext    *context);
 
-typedef struct {
-  float x1, y1, x2, y2;
-  float u1, v1, u2, v2;
-} GdkTexturedQuad;
-
-void                    gdk_gl_texture_quads                    (GdkGLContext    *context,
-                                                                 guint            texture_target,
-                                                                 int              n_quads,
-                                                                 GdkTexturedQuad *quads,
-                                                                 gboolean         flip_colors);
-
-
 G_END_DECLS
 
 #endif /* __GDK_GL_CONTEXT_PRIVATE_H__ */
