@@ -296,7 +296,7 @@ wl_shm_format (void          *data,
 {
   GDK_NOTE (MISC,
             char buf[10];
-            g_message ("supported pixel format %s", get_format_name (format, buf));
+            g_message ("supported pixel format %s (0x%X)", get_format_name (format, buf), (guint) format);
             );
 }
 
@@ -697,11 +697,7 @@ gdk_wayland_display_finalize (GObject *object)
   g_free (display_wayland->cursor_theme_name);
   xkb_context_unref (display_wayland->xkb_context);
 
-  if (display_wayland->cursor_theme)
-    {
-      wl_cursor_theme_destroy (display_wayland->cursor_theme);
-      display_wayland->cursor_theme = NULL;
-    }
+  g_clear_pointer (&display_wayland->cursor_theme, wl_cursor_theme_destroy);
 
   g_list_store_remove_all (display_wayland->monitors);
   g_object_unref (display_wayland->monitors);
@@ -2624,7 +2620,7 @@ gdk_wayland_display_get_output_scale (GdkWaylandDisplay *display_wayland,
  * @display: (type GdkWaylandDisplay): a `GdkDisplay`
  * @global: global interface to query in the registry
  *
- * Returns %TRUE if the the interface was found in the display
+ * Returns %TRUE if the interface was found in the display
  * `wl_registry.global` handler.
  *
  * Returns: %TRUE if the global is offered by the compositor
