@@ -25,13 +25,17 @@
  */
 
 #include "config.h"
-#include <string.h>
 
+#include "gdkwin32dnd.h"
+
+#include "gdkeventsprivate.h"
+#include "gdkseatprivate.h"
 #include "gdk-private.h"
 
-#include <io.h>
 #include <fcntl.h>
+#include <io.h>
 #include <math.h>
+#include <string.h>
 
 /*
  * Support for OLE-2 drag and drop added at Archaeopteryx Software, 2001
@@ -192,7 +196,6 @@
 #define COBJMACROS
 
 #include "gdkdrag.h"
-#include "gdkinternals.h"
 #include "gdkprivate-win32.h"
 #include "gdkwin32.h"
 #include "gdkwin32dnd.h"
@@ -2003,8 +2006,6 @@ static void
 gdk_win32_drag_cancel (GdkDrag             *drag,
                        GdkDragCancelReason  reason)
 {
-  GdkWin32Drag *drag_win32 = GDK_WIN32_DRAG (drag);
-
   const char *reason_str = NULL;
   switch (reason)
     {
@@ -2081,8 +2082,8 @@ gdk_dnd_handle_motion_event (GdkDrag  *drag,
   API_CALL (PostThreadMessage, (clipdrop->dnd_thread_id,
                                 WM_MOUSEMOVE,
                                 key_state,
-                                MAKELPARAM (x_root * drag_win32->scale,
-                                            y_root * drag_win32->scale)));
+                                MAKELPARAM (x_root * drag_win32->scale - _gdk_offset_x,
+                                            y_root * drag_win32->scale - _gdk_offset_y)));
 
   return TRUE;
 }
