@@ -34,7 +34,6 @@
 #include <gdk/win32/gdkclipdrop-win32.h>
 //#include <gdk/win32/gdkselection-win32.h>
 
-#include "gdkinternals.h"
 
 #include "config.h"
 
@@ -259,6 +258,13 @@ extern GdkDisplay       *_gdk_display;
 
 extern GdkDeviceManagerWin32 *_gdk_device_manager;
 
+/* Offsets to add to Windows coordinates (which are relative to the
+ * primary monitor's origin, and thus might be negative for monitors
+ * to the left and/or above the primary monitor) to get GDK
+ * coordinates, which should be non-negative on the whole screen.
+ */
+extern int               _gdk_offset_x, _gdk_offset_y;
+
 extern HDC               _gdk_display_hdc;
 extern HINSTANCE         _gdk_dll_hinstance;
 extern HINSTANCE         _gdk_app_hmodule;
