@@ -210,7 +210,6 @@
 
 #include "gtkbuilderprivate.h"
 
-#include "gdkpixbufutilsprivate.h"
 #include "gtkbuildableprivate.h"
 #include "gtkbuilderlistitemfactory.h"
 #include "gtkbuilderscopeprivate.h"
@@ -226,7 +225,6 @@
 #include "gtktypebuiltins.h"
 #include "gtkicontheme.h"
 #include "gtkiconthemeprivate.h"
-#include "gdkpixbufutilsprivate.h"
 #include "gtkdebug.h"
 
 
@@ -461,7 +459,6 @@ typedef struct
 {
   GPtrArray *names;
   GArray *values;
-  guint len;
 } ObjectProperties;
 
 
@@ -470,8 +467,6 @@ object_properties_init (ObjectProperties *self)
 {
   self->names = NULL;
   self->values = NULL;
-
-  self->len = 0;
 }
 
 static void
@@ -485,8 +480,6 @@ object_properties_destroy (ObjectProperties *self)
 
   if (self->values)
     g_array_unref (self->values);
-
-  self->len = 0;
 }
 
 static void
@@ -495,10 +488,8 @@ object_properties_add (ObjectProperties *self,
                        const GValue     *value)
 {
   if (!self->names)
-    self->names = g_ptr_array_sized_new (8);
-
-  if (!self->values)
     {
+      self->names = g_ptr_array_sized_new (8);
       self->values = g_array_sized_new (FALSE, FALSE, sizeof (GValue), 8);
       g_array_set_clear_func (self->values, (GDestroyNotify) g_value_unset);
     }
@@ -507,8 +498,6 @@ object_properties_add (ObjectProperties *self,
   g_array_append_vals (self->values, value, 1);
 
   g_assert (self->names->len == self->values->len);
-
-  self->len += 1;
 }
 
 static const char *
@@ -548,8 +537,20 @@ gtk_builder_get_parameters (GtkBuilder         *builder,
   for (guint i = 0; i < properties->len; i++)
     {
       PropertyInfo *prop = g_ptr_array_index (properties, i);
-      const char *property_name = g_intern_string (prop->pspec->name);
+      const char *property_name = prop->pspec->name;
       GValue property_value = G_VALUE_INIT;
+      ObjectProperties *params;
+
+      if (prop->applied)
+        continue;
+
+      if ((prop->pspec->flags & filter_flags) != 0)
+        params = filtered_parameters;
+      else
+        params = parameters;
+
+      if (!params)
+        continue;
 
       if (prop->value)
         {
@@ -568,12 +569,12 @@ gtk_builder_get_parameters (GtkBuilder         *builder,
           continue;
         }
       else if (G_IS_PARAM_SPEC_OBJECT (prop->pspec) &&
-          (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GDK_TYPE_PIXBUF) &&
-          (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GDK_TYPE_TEXTURE) &&
-          (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GDK_TYPE_PAINTABLE) &&
-          (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GTK_TYPE_SHORTCUT_TRIGGER) &&
-          (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GTK_TYPE_SHORTCUT_ACTION) &&
-          (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != G_TYPE_FILE))
+               (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GDK_TYPE_PIXBUF) &&
+               (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GDK_TYPE_TEXTURE) &&
+               (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GDK_TYPE_PAINTABLE) &&
+               (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GTK_TYPE_SHORTCUT_TRIGGER) &&
+               (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != GTK_TYPE_SHORTCUT_ACTION) &&
+               (G_PARAM_SPEC_VALUE_TYPE (prop->pspec) != G_TYPE_FILE))
         {
           GObject *object = g_hash_table_lookup (priv->objects,
                                                  g_strstrip (prop->text->str));
@@ -593,6 +594,9 @@ gtk_builder_get_parameters (GtkBuilder         *builder,
                   continue;
                 }
               /* Delay setting property */
+
+              prop->applied = TRUE;
+
               property = g_slice_new (DelayedProperty);
               property->pspec = prop->pspec;
               property->object = g_strdup (object_name);
@@ -617,12 +621,12 @@ gtk_builder_get_parameters (GtkBuilder         *builder,
           continue;
         }
 
-      if ((prop->pspec->flags & filter_flags) != 0 && filtered_parameters)
-        object_properties_add (filtered_parameters, property_name, &property_value);
-      else if ((prop->pspec->flags & filter_flags) == 0 && parameters)
-        object_properties_add (parameters, property_name, &property_value);
-      else
-        g_value_unset (&property_value);
+      /* At this point, property_value has been set, and we need to either
+       * copy it to one of the two arrays, or unset it.
+       */
+      g_assert (G_IS_VALUE (&property_value));
+      object_properties_add (params, property_name, &property_value);
+      prop->applied = TRUE;
     }
 }
 
@@ -693,7 +697,7 @@ _gtk_builder_add_object (GtkBuilder  *builder,
   g_hash_table_insert (priv->objects, g_strdup (id), g_object_ref (object));
 }
 
-static void
+void
 gtk_builder_take_bindings (GtkBuilder *builder,
                            GObject    *target,
                            GSList     *bindings)
@@ -749,9 +753,6 @@ _gtk_builder_construct (GtkBuilder  *builder,
   ObjectProperties parameters, construct_parameters;
   GObject *obj;
   int i;
-  GtkBuildableIface *iface;
-  gboolean custom_set_property;
-  GtkBuildable *buildable;
   GParamFlags param_filter_flags;
 
   g_assert (info->type != G_TYPE_INVALID);
@@ -819,7 +820,7 @@ _gtk_builder_construct (GtkBuilder  *builder,
                                            builder,
                                            info->id);
       g_assert (obj != NULL);
-      if (construct_parameters.len > 0)
+      if (construct_parameters.names->len > 0)
         g_warning ("Can't pass in construct-only parameters to %s", info->id);
     }
   else if (info->parent &&
@@ -834,7 +835,7 @@ _gtk_builder_construct (GtkBuilder  *builder,
           object_properties_destroy (&construct_parameters);
           return NULL;
         }
-      if (construct_parameters.len > 0)
+      if (construct_parameters.names)
         g_warning ("Can't pass in construct-only parameters to %s", childname);
       g_object_ref (obj);
     }
@@ -842,9 +843,9 @@ _gtk_builder_construct (GtkBuilder  *builder,
     {
       ensure_special_construct_parameters (builder, info->type, &construct_parameters);
 
-      if (construct_parameters.len > 0)
+      if (construct_parameters.names)
         obj = g_object_new_with_properties (info->type,
-                                            construct_parameters.len,
+                                            construct_parameters.names->len,
                                             (const char **) construct_parameters.names->pdata,
                                             (GValue *) construct_parameters.values->data);
       else
@@ -867,40 +868,61 @@ _gtk_builder_construct (GtkBuilder  *builder,
     }
   object_properties_destroy (&construct_parameters);
 
-  custom_set_property = FALSE;
-  buildable = NULL;
-  iface = NULL;
-  if (GTK_IS_BUILDABLE (obj))
+  if (parameters.names)
     {
-      buildable = GTK_BUILDABLE (obj);
-      iface = GTK_BUILDABLE_GET_IFACE (obj);
-      if (iface->set_buildable_property)
-        custom_set_property = TRUE;
-    }
+      GtkBuildableIface *iface = NULL;
+      gboolean custom_set_property = FALSE;
+      GtkBuildable *buildable = NULL;
 
-  for (i = 0; i < parameters.len; i++)
-    {
-      const char *name = object_properties_get_name (&parameters, i);
-      const GValue *value = object_properties_get_value (&parameters, i);
+      if (GTK_IS_BUILDABLE (obj))
+        {
+          buildable = GTK_BUILDABLE (obj);
+          iface = GTK_BUILDABLE_GET_IFACE (obj);
+          if (iface->set_buildable_property)
+            custom_set_property = TRUE;
+        }
 
       if (custom_set_property)
-        iface->set_buildable_property (buildable, builder, name, value);
-      else
-        g_object_set_property (obj, name, value);
-
-#ifdef G_ENABLE_DEBUG
-      if (GTK_DEBUG_CHECK (BUILDER))
         {
-          char *str = g_strdup_value_contents (value);
-          g_message ("set %s: %s = %s", info->id, name, str);
-          g_free (str);
-        }
-#endif
-    }
-  object_properties_destroy (&parameters);
+          for (i = 0; i < parameters.names->len; i++)
+            {
+              const char *name = object_properties_get_name (&parameters, i);
+              const GValue *value = object_properties_get_value (&parameters, i);
 
-  if (info->bindings)
-    gtk_builder_take_bindings (builder, obj, info->bindings);
+              iface->set_buildable_property (buildable, builder, name, value);
+#ifdef G_ENABLE_DEBUG
+              if (GTK_DEBUG_CHECK (BUILDER))
+                {
+                  char *str = g_strdup_value_contents (value);
+                  g_message ("set %s: %s = %s", info->id, name, str);
+                  g_free (str);
+                }
+#endif
+            }
+        }
+      else
+        {
+          g_object_setv (obj,
+                         parameters.names->len,
+                         (const char **) parameters.names->pdata,
+                         (GValue *) parameters.values->data);
+#ifdef G_ENABLE_DEBUG
+          if (GTK_DEBUG_CHECK (BUILDER))
+            {
+              for (i = 0; i < parameters.names->len; i++)
+                {
+                  const char *name = object_properties_get_name (&parameters, i);
+                  const GValue *value = object_properties_get_value (&parameters, i);
+                  char *str = g_strdup_value_contents (value);
+                  g_message ("set %s: %s = %s", info->id, name, str);
+                  g_free (str);
+                }
+            }
+#endif
+        }
+    }
+
+  object_properties_destroy (&parameters);
 
   /* put it in the hash table. */
   _gtk_builder_add_object (builder, info->id, obj);
@@ -917,10 +939,6 @@ _gtk_builder_apply_properties (GtkBuilder  *builder,
                                GError     **error)
 {
   ObjectProperties parameters;
-  GtkBuildableIface *iface;
-  GtkBuildable *buildable;
-  gboolean custom_set_property;
-  int i;
 
   g_assert (info->object != NULL);
   g_assert (info->type != G_TYPE_INVALID);
@@ -934,36 +952,60 @@ _gtk_builder_apply_properties (GtkBuilder  *builder,
                               G_PARAM_CONSTRUCT_ONLY,
                               &parameters, NULL);
 
-
-  custom_set_property = FALSE;
-  buildable = NULL;
-  iface = NULL;
-  if (GTK_IS_BUILDABLE (info->object))
+  if (parameters.names)
     {
-      buildable = GTK_BUILDABLE (info->object);
-      iface = GTK_BUILDABLE_GET_IFACE (info->object);
-      if (iface->set_buildable_property)
-        custom_set_property = TRUE;
-    }
+      GtkBuildableIface *iface = NULL;
+      GtkBuildable *buildable = NULL;
+      gboolean custom_set_property = FALSE;
+      int i;
 
-  for (i = 0; i < parameters.len; i++)
-    {
-      const char *name = object_properties_get_name (&parameters, i);
-      const GValue *value = object_properties_get_value (&parameters, i);
-      if (custom_set_property)
-        iface->set_buildable_property (buildable, builder, name, value);
-      else
-        g_object_set_property (info->object, name, value);
-
-#ifdef G_ENABLE_DEBUG
-      if (GTK_DEBUG_CHECK (BUILDER))
+      if (GTK_IS_BUILDABLE (info->object))
         {
-          char *str = g_strdup_value_contents (value);
-          g_message ("set %s: %s = %s", info->id, name, str);
-          g_free (str);
+          buildable = GTK_BUILDABLE (info->object);
+          iface = GTK_BUILDABLE_GET_IFACE (info->object);
+          if (iface->set_buildable_property)
+            custom_set_property = TRUE;
         }
+
+      if (custom_set_property)
+        {
+          for (i = 0; i < parameters.names->len; i++)
+            {
+              const char *name = object_properties_get_name (&parameters, i);
+              const GValue *value = object_properties_get_value (&parameters, i);
+              iface->set_buildable_property (buildable, builder, name, value);
+#ifdef G_ENABLE_DEBUG
+              if (GTK_DEBUG_CHECK (BUILDER))
+                {
+                  char *str = g_strdup_value_contents (value);
+                  g_message ("set %s: %s = %s", info->id, name, str);
+                  g_free (str);
+                }
 #endif
+            }
+        }
+      else
+        {
+          g_object_setv (info->object,
+                         parameters.names->len,
+                         (const char **) parameters.names->pdata,
+                         (GValue *) parameters.values->data);
+#ifdef G_ENABLE_DEBUG
+          if (GTK_DEBUG_CHECK (BUILDER))
+            {
+              for (i = 0; i < parameters.names->len; i++)
+                {
+                  const char *name = object_properties_get_name (&parameters, i);
+                  const GValue *value = object_properties_get_value (&parameters, i);
+                  char *str = g_strdup_value_contents (value);
+                  g_message ("set %s: %s = %s", info->id, name, str);
+                  g_free (str);
+                }
+            }
+#endif
+        }
     }
+
   object_properties_destroy (&parameters);
 }
 
@@ -1921,9 +1963,9 @@ gboolean
 _gtk_builder_finish (GtkBuilder  *builder,
                      GError     **error)
 {
-  return gtk_builder_apply_delayed_properties (builder, error)
-      && gtk_builder_create_bindings (builder, error)
-      && gtk_builder_connect_signals (builder, error);
+  return gtk_builder_apply_delayed_properties (builder, error) &&
+         gtk_builder_create_bindings (builder, error) &&
+         gtk_builder_connect_signals (builder, error);
 }
 
 /**
@@ -1969,7 +2011,7 @@ gtk_builder_value_from_string (GtkBuilder   *builder,
     {
       gunichar c;
       g_value_init (value, G_TYPE_UINT);
-      c = g_utf8_get_char_validated (string, strlen (string));
+      c = g_utf8_get_char_validated (string, -1);
       if (c != 0 && c != (gunichar)-1 && c != (gunichar)-2)
         g_value_set_uint (value, c);
       return TRUE;
@@ -2176,7 +2218,7 @@ gtk_builder_value_from_string_type (GtkBuilder   *builder,
       {
         guint flags_value;
 
-        if (!_gtk_builder_flags_from_string (type, NULL, string, &flags_value, error))
+        if (!_gtk_builder_flags_from_string (type, string, &flags_value, error))
           {
             ret = FALSE;
             break;
@@ -2292,9 +2334,63 @@ gtk_builder_value_from_string_type (GtkBuilder   *builder,
       break;
     case G_TYPE_OBJECT:
     case G_TYPE_INTERFACE:
-      if (G_VALUE_HOLDS (value, GDK_TYPE_PIXBUF) ||
-          G_VALUE_HOLDS (value, GDK_TYPE_PAINTABLE) ||
+      if (G_VALUE_HOLDS (value, GDK_TYPE_PAINTABLE) ||
           G_VALUE_HOLDS (value, GDK_TYPE_TEXTURE))
+        {
+          GObject *object = g_hash_table_lookup (priv->objects, string);
+          char *filename;
+          GError *tmp_error = NULL;
+          GdkTexture *texture = NULL;
+
+          if (object)
+            {
+              if (g_type_is_a (G_OBJECT_TYPE (object), G_VALUE_TYPE (value)))
+                {
+                  g_value_set_object (value, object);
+                  return TRUE;
+                }
+              else
+                {
+                  g_set_error (error,
+                               GTK_BUILDER_ERROR,
+                               GTK_BUILDER_ERROR_INVALID_VALUE,
+                               "Could not load image '%s': "
+                               " '%s' is already used as object id for a %s",
+                               string, string, G_OBJECT_TYPE_NAME (object));
+                  return FALSE;
+                }
+            }
+
+          filename = _gtk_builder_get_resource_path (builder, string);
+          if (filename != NULL)
+            {
+              texture = gdk_texture_new_from_resource (filename);
+            }
+          else
+            {
+              GFile *file;
+
+              filename = _gtk_builder_get_absolute_filename (builder, string);
+              file = g_file_new_for_path (filename);
+              texture = gdk_texture_new_from_file (file, &tmp_error);
+              g_object_unref (file);
+            }
+
+          g_free (filename);
+
+          if (!texture)
+            {
+              g_warning ("Could not load image '%s': %s", string, tmp_error->message);
+              g_error_free (tmp_error);
+
+              texture = gdk_texture_new_from_resource (IMAGE_MISSING_RESOURCE_PATH);
+            }
+
+          g_value_take_object (value, texture);
+
+          ret = TRUE;
+        }
+      else if (G_VALUE_HOLDS (value, GDK_TYPE_PIXBUF))
         {
           char *filename;
           GError *tmp_error = NULL;
@@ -2340,28 +2436,13 @@ gtk_builder_value_from_string_type (GtkBuilder   *builder,
 
           if (pixbuf == NULL)
             {
-              g_warning ("Could not load image '%s': %s",
-                         string, tmp_error->message);
+              g_warning ("Could not load image '%s': %s", string, tmp_error->message);
               g_error_free (tmp_error);
 
-              pixbuf = _gdk_pixbuf_new_from_resource (IMAGE_MISSING_RESOURCE_PATH, "png", NULL);
+              pixbuf = gdk_pixbuf_new_from_resource (IMAGE_MISSING_RESOURCE_PATH, NULL);
             }
 
-          if (pixbuf)
-            {
-              if (G_VALUE_HOLDS (value, GDK_TYPE_TEXTURE) ||
-                  G_VALUE_HOLDS (value, GDK_TYPE_PAINTABLE))
-                {
-                  GdkTexture *texture = gdk_texture_new_for_pixbuf (pixbuf);
-                  g_value_set_object (value, texture);
-                  g_object_unref (texture);
-                }
-              else
-                {
-                  g_value_set_object (value, pixbuf);
-                }
-              g_object_unref (G_OBJECT (pixbuf));
-            }
+          g_value_take_object (value, pixbuf);
 
           g_free (filename);
 
@@ -2502,9 +2583,9 @@ _gtk_builder_enum_from_string (GType         type,
   else
     {
       eclass = g_type_class_ref (type);
-      ev = g_enum_get_value_by_name (eclass, string);
+      ev = g_enum_get_value_by_nick (eclass, string);
       if (!ev)
-        ev = g_enum_get_value_by_nick (eclass, string);
+        ev = g_enum_get_value_by_name (eclass, string);
 
       if (ev)
         *enum_value = ev->value;
@@ -2526,14 +2607,13 @@ _gtk_builder_enum_from_string (GType         type,
 
 gboolean
 _gtk_builder_flags_from_string (GType         type,
-                                GFlagsValue  *aliases,
                                 const char   *string,
                                 guint        *flags_value,
                                 GError      **error)
 {
   GFlagsClass *fclass;
   char *endptr, *prevptr;
-  guint i, j, k, value;
+  guint i, j, value;
   char *flagstr;
   GFlagsValue *fv;
   const char *flag;
@@ -2596,18 +2676,6 @@ _gtk_builder_flags_from_string (GType         type,
 
               fv = NULL;
 
-              if (aliases)
-                {
-                  for (k = 0; aliases[k].value_nick; k++)
-                    {
-                      if (g_ascii_strcasecmp (aliases[k].value_nick, flag) == 0)
-                        {
-                          fv = &aliases[k];
-                          break;
-                        }
-                    }
-                }
-
               if (!fv)
                 fv = g_flags_get_value_by_name (fclass, flag);
 
@@ -2748,7 +2816,7 @@ _gtk_builder_get_template_type (GtkBuilder *builder)
  * Creates a closure to invoke the function called @function_name.
  *
  * This is using the create_closure() implementation of @builder's
- * [iface@Gtk.BuilderScope].
+ * [class@Gtk.BuilderScope].
  *
  * If no closure could be created, %NULL will be returned and @error
  * will be set.
