@@ -33,6 +33,7 @@
 #include "gtkcssstringvalueprivate.h"
 #include "gtkcssfontvariationsvalueprivate.h"
 #include "gtkcssfontfeaturesvalueprivate.h"
+#include "gtkcsslineheightvalueprivate.h"
 #include "gtkcssstylepropertyprivate.h"
 #include "gtkcsstransitionprivate.h"
 #include "gtkstyleanimationprivate.h"
@@ -117,12 +118,16 @@ gtk_css_style_get_value (GtkCssStyle *style,
       return style->font->font_stretch;
     case GTK_CSS_PROPERTY_LETTER_SPACING:
       return style->font->letter_spacing;
+    case GTK_CSS_PROPERTY_LINE_HEIGHT:
+      return style->font->line_height;
     case GTK_CSS_PROPERTY_TEXT_DECORATION_LINE:
       return style->font_variant->text_decoration_line;
     case GTK_CSS_PROPERTY_TEXT_DECORATION_COLOR:
       return style->font_variant->text_decoration_color ? style->font_variant->text_decoration_color : style->core->color;
     case GTK_CSS_PROPERTY_TEXT_DECORATION_STYLE:
       return style->font_variant->text_decoration_style;
+    case GTK_CSS_PROPERTY_TEXT_TRANSFORM:
+      return style->font_variant->text_transform;
     case GTK_CSS_PROPERTY_FONT_KERNING:
       return style->font_variant->font_kerning;
     case GTK_CSS_PROPERTY_FONT_VARIANT_LIGATURES:
@@ -397,6 +402,24 @@ get_pango_underline_from_style (GtkTextDecorationStyle style)
   g_return_val_if_reached (PANGO_UNDERLINE_SINGLE);
 }
 
+PangoTextTransform
+gtk_css_style_get_pango_text_transform (GtkCssStyle *style)
+{
+  switch (_gtk_css_text_transform_value_get (style->font_variant->text_transform))
+    {
+    case GTK_CSS_TEXT_TRANSFORM_NONE:
+      return PANGO_TEXT_TRANSFORM_NONE;
+    case GTK_CSS_TEXT_TRANSFORM_LOWERCASE:
+      return PANGO_TEXT_TRANSFORM_LOWERCASE;
+    case GTK_CSS_TEXT_TRANSFORM_UPPERCASE:
+      return PANGO_TEXT_TRANSFORM_UPPERCASE;
+    case GTK_CSS_TEXT_TRANSFORM_CAPITALIZE:
+      return PANGO_TEXT_TRANSFORM_CAPITALIZE;
+    default:
+      return PANGO_TEXT_TRANSFORM_NONE;
+    }
+}
+
 static PangoOverline
 get_pango_overline_from_style (GtkTextDecorationStyle style)
 {
@@ -640,6 +663,18 @@ gtk_css_style_get_pango_attributes (GtkCssStyle *style)
       attrs = add_pango_attr (attrs, pango_attr_letter_spacing_new (letter_spacing * PANGO_SCALE));
     }
 
+  /* line-height */
+  {
+    double height = gtk_css_line_height_value_get (style->font->line_height);
+    if (height != 0.0)
+      {
+        if (gtk_css_number_value_get_dimension (style->font->line_height) == GTK_CSS_DIMENSION_LENGTH)
+          attrs = add_pango_attr (attrs, pango_attr_line_height_new_absolute (height * PANGO_SCALE));
+        else
+          attrs = add_pango_attr (attrs, pango_attr_line_height_new (height));
+      }
+   }
+
   /* OpenType features */
   {
     char *font_features = gtk_css_style_compute_font_features (style);
@@ -651,6 +686,14 @@ gtk_css_style_get_pango_attributes (GtkCssStyle *style)
       }
   }
 
+  /* text-transform */
+  {
+    PangoTextTransform transform = gtk_css_style_get_pango_text_transform (style);
+
+    if (transform != PANGO_TEXT_TRANSFORM_NONE)
+      attrs = add_pango_attr (attrs, pango_attr_text_transform_new (transform));
+  }
+
   return attrs;
 }
 
