@@ -49,7 +49,7 @@ struct _GtkEditableInterface
 
   /* signals */
   void (* insert_text)              (GtkEditable    *editable,
-                                     const gchar    *text,
+                                     const char     *text,
                                      int             length,
                                      int            *position);
   void (* delete_text)              (GtkEditable    *editable,
@@ -87,7 +87,7 @@ void     gtk_editable_set_text             (GtkEditable *editable,
 GDK_AVAILABLE_IN_ALL
 char *   gtk_editable_get_chars            (GtkEditable *editable,
                                             int          start_pos,
-                                            int          end_pos);
+                                            int          end_pos) G_GNUC_MALLOC;
 GDK_AVAILABLE_IN_ALL
 void     gtk_editable_insert_text          (GtkEditable *editable,
                                             const char  *text,
@@ -162,6 +162,8 @@ GDK_AVAILABLE_IN_ALL
 guint        gtk_editable_install_properties    (GObjectClass *object_class,
                                                  guint         first_prop);
 GDK_AVAILABLE_IN_ALL
+GtkEditable *gtk_editable_get_delegate          (GtkEditable *editable);
+GDK_AVAILABLE_IN_ALL
 void         gtk_editable_init_delegate         (GtkEditable  *editable);
 GDK_AVAILABLE_IN_ALL
 void         gtk_editable_finish_delegate       (GtkEditable  *editable);
