@@ -490,6 +490,16 @@ gtk_search_entry_accessible_init (GtkAccessibleInterface *iface)
   iface->get_platform_state = gtk_search_entry_accessible_get_platform_state;
 }
 
+static void
+gtk_search_entry_icon_press (GtkGestureClick *press,
+                             int              n_press,
+                             double           x,
+                             double           y,
+                             GtkSearchEntry  *entry)
+{
+  gtk_gesture_set_state (GTK_GESTURE (press), GTK_EVENT_SEQUENCE_CLAIMED);
+}
+
 static void
 gtk_search_entry_icon_release (GtkGestureClick *press,
                                int              n_press,
@@ -569,11 +579,21 @@ activate_cb (GtkText  *text,
   g_signal_emit (data, signals[ACTIVATE], 0);
 }
 
+static void
+catchall_click_press (GtkGestureClick *gesture,
+                      int              n_press,
+                      double           x,
+                      double           y,
+                      gpointer         user_data)
+{
+  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
+}
+
 static void
 gtk_search_entry_init (GtkSearchEntry *entry)
 {
   GtkWidget *icon;
-  GtkGesture *press;
+  GtkGesture *press, *catchall;
 
   /* The search icon is purely presentational */
   icon = g_object_new (GTK_TYPE_IMAGE,
@@ -601,9 +621,16 @@ gtk_search_entry_init (GtkSearchEntry *entry)
   gtk_widget_set_child_visible (entry->icon, FALSE);
 
   press = gtk_gesture_click_new ();
+  g_signal_connect (press, "pressed", G_CALLBACK (gtk_search_entry_icon_press), entry);
   g_signal_connect (press, "released", G_CALLBACK (gtk_search_entry_icon_release), entry);
   gtk_widget_add_controller (entry->icon, GTK_EVENT_CONTROLLER (press));
 
+  catchall = gtk_gesture_click_new ();
+  g_signal_connect (catchall, "pressed",
+                    G_CALLBACK (catchall_click_press), entry);
+  gtk_widget_add_controller (GTK_WIDGET (entry),
+                             GTK_EVENT_CONTROLLER (catchall));
+
   gtk_widget_add_css_class (GTK_WIDGET (entry), I_("search"));
 }
 
