@@ -51,7 +51,7 @@
  * It functions in a similar way to a cairo context, and maintains a stack
  * of render nodes and their associated transformations.
  *
- * The node at the top of the stack is the the one that gtk_snapshot_append_…
+ * The node at the top of the stack is the one that gtk_snapshot_append_…
  * functions operate on. Use the gtk_snapshot_push_… functions and
  * gtk_snapshot_pop() to change the current node.
  *
@@ -520,6 +520,9 @@ gtk_snapshot_collect_blur (GtkSnapshot      *snapshot,
   if (radius == 0.0)
     return node;
 
+  if (radius < 0)
+    return node;
+
   blur_node = gsk_blur_node_new (node, radius);
 
   gsk_render_node_unref (node);
@@ -530,7 +533,7 @@ gtk_snapshot_collect_blur (GtkSnapshot      *snapshot,
 /**
  * gtk_snapshot_push_blur:
  * @snapshot: a `GtkSnapshot`
- * @radius: the blur radius to use
+ * @radius: the blur radius to use. Must be positive
  *
  * Blurs an image.
  *
@@ -1125,7 +1128,7 @@ gtk_snapshot_clear_shadow (GtkSnapshotState *state)
 /**
  * gtk_snapshot_push_shadow:
  * @snapshot: a `GtkSnapshot`
- * @shadow: the first shadow specification
+ * @shadow: (array length=n_shadows): the first shadow specification
  * @n_shadows: number of shadow specifications
  *
  * Applies a shadow to an image.
