@@ -34,6 +34,7 @@
 #include "gtkbinlayout.h"
 #include "gtkmediafileprivate.h"
 
+#include "gdk/gdkdebug.h"
 
 #ifdef GDK_WINDOWING_X11
 #include "x11/gdkx.h"
@@ -157,8 +158,6 @@ init_version (GtkInspectorGeneral *gen)
     renderer = "Vulkan";
   else if (strcmp (G_OBJECT_TYPE_NAME (gsk_renderer), "GskGLRenderer") == 0)
     renderer = "GL";
-  else if (strcmp (G_OBJECT_TYPE_NAME (gsk_renderer), "GskNglRenderer") == 0)
-    renderer = "NGL";
   else if (strcmp (G_OBJECT_TYPE_NAME (gsk_renderer), "GskCairoRenderer") == 0)
     renderer = "Cairo";
   else
@@ -371,10 +370,10 @@ init_gl (GtkInspectorGeneral *gen)
       char *version;
 
       gl_version = epoxy_gl_version ();
-      version = g_strdup_printf ("WGL %d.%d", gl_version / 10, gl_version % 10, NULL);
+      version = g_strdup_printf ("WGL %d.%d", gl_version / 10, gl_version % 10);
       gtk_label_set_text (GTK_LABEL (gen->gl_version), version);
       g_free (version);
-      gtk_label_set_text (GTK_LABEL (gen->gl_vendor), glGetString (GL_VENDOR));
+      gtk_label_set_text (GTK_LABEL (gen->gl_vendor), (const char *) glGetString (GL_VENDOR));
 
       append_wgl_extension_row (gen, "WGL_EXT_create_context");
       append_wgl_extension_row (gen, "WGL_EXT_swap_control");
